/* eslint-disable camelcase */
import { InitData } from './interface'
/**
 *@description 富文本编辑器字体配置
 */
export const FONT_FORMATS = `思源宋体=tonjay;
思源黑体=Noto Sans SC;
微软雅黑=Microsoft YaHei;
Helvetica=Helvetica;
Source Han Sans CN = Source Han Sans CN;
sans-serif=sans-serif;
苹方=PingFang SC;
宋体=simsun;
黑体=SimHei,sans-serif;Arial=arial,helvetica,sans-serif;
Arial=arial black,avant garde;
Book Antiqua=book antiqua,palatino;
Tahoma = Tahoma;
`
/**
 * @description 富文本编辑器插件地址配置
 */
const defaultPlugins = {
  advlist: new URL(`./tinymce/js/tinymce/plugins/advlist/plugin.min.js`, import.meta.url).href,
  anchor: new URL(`./tinymce/js/tinymce/plugins/anchor/plugin.min.js`, import.meta.url).href,
  autolink: new URL(`./tinymce/js/tinymce/plugins/autolink/plugin.min.js`, import.meta.url).href,
  autosave: new URL(`./tinymce/js/tinymce/plugins/autosave/plugin.min.js`, import.meta.url).href,
  charmap: new URL(`./tinymce/js/tinymce/plugins/charmap/plugin.min.js`, import.meta.url).href,
  codesample: new URL(`./tinymce/js/tinymce/plugins/codesample/plugin.min.js`, import.meta.url)
    .href,
  directionality: new URL(
    `./tinymce/js/tinymce/plugins/directionality/plugin.min.js`,
    import.meta.url
  ).href,
  fullscreen: new URL(`./tinymce/js/tinymce/plugins/fullscreen/plugin.min.js`, import.meta.url)
    .href,
  hr: new URL(`./tinymce/js/tinymce/plugins/hr/plugin.min.js`, import.meta.url).href,
  image: new URL(`./tinymce/js/tinymce/plugins/image/plugin.min.js`, import.meta.url).href,
  insertdatetime: new URL(
    `./tinymce/js/tinymce/plugins/insertdatetime/plugin.min.js`,
    import.meta.url
  ).href,
  link: new URL(`./tinymce/js/tinymce/plugins/link/plugin.min.js`, import.meta.url).href,
  lists: new URL(`./tinymce/js/tinymce/plugins/lists/plugin.min.js`, import.meta.url).href,
  nonbreaking: new URL(`./tinymce/js/tinymce/plugins/nonbreaking/plugin.min.js`, import.meta.url)
    .href,
  pagebreak: new URL(`./tinymce/js/tinymce/plugins/pagebreak/plugin.min.js`, import.meta.url).href,
  paste: new URL(`./tinymce/js/tinymce/plugins/paste/plugin.min.js`, import.meta.url).href,
  searchreplace: new URL(
    `./tinymce/js/tinymce/plugins/searchreplace/plugin.min.js`,
    import.meta.url
  ).href,
  table: new URL(`./tinymce/js/tinymce/plugins/table/plugin.min.js`, import.meta.url).href,
  template: new URL(`./tinymce/js/tinymce/plugins/template/plugin.min.js`, import.meta.url).href,
  textpattern: new URL(`./tinymce/js/tinymce/plugins/textpattern/plugin.min.js`, import.meta.url)
    .href,
  visualblocks: new URL(`./tinymce/js/tinymce/plugins/visualblocks/plugin.min.js`, import.meta.url)
    .href,
  visualchars: new URL(`./tinymce/js/tinymce/plugins/visualchars/plugin.min.js`, import.meta.url)
    .href,
  wordcount: new URL(`./tinymce/js/tinymce/plugins/wordcount/plugin.min.js`, import.meta.url).href,
}
/**
 *@description 富文本编辑器工具栏配置
 */
//  media
export const toolBar = [
  'fullscreen file image undo redo restoredraft | cut copy paste pastetext | forecolor backcolor bold italic underline strikethrough link anchor | alignleft aligncenter alignright alignjustify outdent indent | bullist numlist | blockquote subscript superscript removeformat ',
  'styleselect fontselect fontsizeselect |  table axupimgs emoticons charmap hr pagebreak insertdatetime  selectall visualblocks searchreplace | indent2em lineheight formatpainter',
]

/**
 *@description 富文本初始化函数
 */
export const tinymceInit = (initData: InitData) => {
  return {
    //语言类型
    language: 'zh_CN',
    // 语言路径
    language_url: initData.languageUrl,
    // 主题
    theme_url: initData.themeUrl,
    // 皮肤
    skin_url: initData.skinUrl,
    // 图标
    icons_url: initData.iconsUrl,
    // 内容css
    content_css: initData.contentCss,
    //选择器
    selector: initData.selector,
    // 插件地址
    external_plugins: Object.assign(defaultPlugins, initData.externalPlugins),
    // textarea中的提示信息
    placeholder: initData.placeholder,
    // 注：引入autoresize插件时，此属性失效
    height: initData.height,
    // 注：引入autoresize插件时，此属性失效
    width: initData.width,
    // 富文本最小宽度
    min_width: initData.minWidth,
    // 富文本最小高度
    min_height: initData.minHeight,
    // 富文本最大宽度
    max_width: initData.maxWidth,
    // 富文本最大高度
    max_height: initData.maxHeight,
    // 编辑器宽高是否可变，false-否,true-高可变，'both'-宽高均可，注意引号
    resize: initData.resize,
    // tiny技术支持信息是否显示
    branding: false,
    // 最下方的元素路径和字数统计那一栏是否显示
    statusbar: initData.statusbar,
    // 元素路径是否显示
    elementpath: initData.elementPath,
    //字体样式
    font_formats: initData.fontFormats,
    // 字体大小
    fontsize_formats: initData.fontsizeFormats,
    // 工具栏
    toolbar: initData.toolbar,
    // 菜单栏
    menubar: initData.menubar,
    // 向图像添加自定义样式、间距和边框
    image_advtab: true,
    // 图片是否可粘贴
    paste_data_images: true,
    paste_auto_cleanup_on_paste: true,
    paste_remove_styles: true,
    paste_remove_styles_if_webkit: true,
    paste_strip_class_attributes: true,
    paste_as_text: true,
    paste_text_sticky: true,
    // 富文本加载完成回调
    init_instance_callback: initData.finishCallBack,
    //此处为图片上传处理函数
    images_upload_handler: initData.imagesUploadHandler
      ? initData.imagesUploadHandler
      : (
          blobInfo: { blob: () => Blob },
          success: (e: string) => void,
          failure: (arg0: string) => void
        ) => {
          if (blobInfo.blob().size / 1024 / 1024 > 2) {
            failure('上传失败，图片大小请控制在 2M 以内')
          } else {
            let params = new FormData()
            // 设置上传文件名
            params.append(initData.name, blobInfo.blob())
            // 设置上传文件参数
            for (const key in initData.data) {
              params.append(key, initData.data[key])
            }
            const xhr = new XMLHttpRequest()
            xhr.withCredentials = false
            // 资源上传服务
            xhr.open('POST', initData.action)
            // 设置请求头
            Object.keys(initData.headers).forEach((item) => {
              xhr.setRequestHeader(item, initData.headers[item])
            })
            xhr.onload = function () {
              if (xhr.status != 200) {
                failure('HTTP Error: ' + xhr.status)
                return
              }
              const json = JSON.parse(xhr.responseText)
              if (json.rt_code === 0 || json.rt_code === 200) {
                success(json.data?.[initData.backFileUrlName])
              } else {
                failure('上传出错，服务器开小差了!')
              }
            }
            xhr.send(params)
          }
        },
    // file image media分别对应三个类型文件的上传：link插件，image和axupimgs插件，media插件。想屏蔽某个插件的上传就去掉对应的参数
    file_picker_types: initData.filePickerTypes,
    // 文件上传处理函数
    file_picker_callback: initData.filePickerCallback
      ? initData.filePickerCallback
      : (
          callback: (e: string, t: { title: string; text: string }) => void,
          meta: { filetype: string }
        ) => {
          //限制文件的上传类型
          let acceptType = initData.imageAccept
          const { filetype } = meta
          switch (filetype) {
            case 'image':
              acceptType = initData.imageAccept
              break
            case 'media':
              acceptType = initData.mediaAccept
              break
            case 'file':
              acceptType = initData.fileAccept
              break
          }
          let inputElem = document.createElement('input') //创建文件选择
          inputElem.setAttribute('type', 'file')
          inputElem.setAttribute('accept', acceptType.join(', '))
          inputElem.click()
          inputElem.onchange = () => {
            const filesList = inputElem?.files
            if (!filesList) return
            // 获取文件信息
            let file = filesList[0]
            if (file) {
              if (
                file.type.slice(0, 5) == 'image' &&
                file.size / 1024 / 1024 > initData.imageSize
              ) {
                alert(`上传失败，图片大小请控制在${initData.imageSize}M以内`)
              } else if (
                file.type.slice(0, 5) == 'video' &&
                file.size / 1024 / 1024 > initData.mediaSize
              ) {
                alert(`上传失败，媒体文件大小请控制在${initData.mediaSize}M以内`)
              } else if (file.size / 1024 / 1024 > initData.fileSize) {
                alert(`上传失败，文件大小请控制在${initData.fileSize}M以内`)
              } else {
                let params = new FormData()
                // 设置上传文件名
                params.append(initData.name, file)
                // 设置上传文件参数
                for (const key in initData.data) {
                  params.append(key, initData.data[key])
                }
                const xhr = new XMLHttpRequest()
                xhr.withCredentials = false
                // 资源上传服务
                xhr.open('POST', initData.action)
                // 设置请求头
                Object.keys(initData.headers).forEach((item) => {
                  xhr.setRequestHeader(item, initData.headers[item])
                })
                xhr.onload = async function () {
                  if (xhr.status != 200) {
                    alert('HTTP Error: ' + xhr.status)
                    return
                  }
                  const json = JSON.parse(xhr.responseText)
                  if (json.rt_code === 0 || json.rt_code === 200) {
                    callback(json.data?.[initData.backFileUrlName], {
                      title: file.name,
                      text: json.data?.[initData.backFileUrlName],
                    })
                  } else {
                    alert('上传出错，服务器开小差了!')
                  }
                }
                xhr.send(params)
              }
            } else {
              alert('上传失败')
            }
          }
        },
  }
}
